<?php
/**
 * Plugin main class.
 */

namespace WPDesk\GatewayWPPay;

use WPDesk\GatewayWPPay\BlueMediaApi\BlueMediaClientFactory;
use WPDesk\GatewayWPPay\WooCommerceGateway\BlikZeroEmbedGateway;
use WPDesk\GatewayWPPay\WooCommerceGateway\CardEmbedGateway;
use WPDesk\GatewayWPPay\WooCommerceGateway\StandardPaymentGateway;
use WPPayVendor\WPDesk_Plugin_Info;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\NullLogger;
use WPPayVendor\WPDesk\PluginBuilder\Plugin\AbstractPlugin;
use WPPayVendor\WPDesk\PluginBuilder\Plugin\HookableCollection;
use WPPayVendor\WPDesk\PluginBuilder\Plugin\HookableParent;

/**
 * Main plugin class. The most important flow decisions are made here.
 */
class Plugin extends AbstractPlugin implements LoggerAwareInterface, HookableCollection {

	use LoggerAwareTrait;
	use HookableParent;

	private $scripts_version = 3;

	const WPPAY_REGISTER_URL = 'https://wpde.sk/wppay-plugin-rejestracja';
	const NONCE_AJAX_ACTION_GET_SINGLE_CURRENCY = 'nonce_wppay_get_single_currency';

	/**
	 * Plugin constructor.
	 *
	 * @param WPDesk_Plugin_Info $plugin_info Plugin info.
	 */
	public function __construct( WPDesk_Plugin_Info $plugin_info ) {
		parent::__construct( $plugin_info );
		$this->setLogger( new NullLogger() );

		$this->plugin_url       = $this->plugin_info->get_plugin_url();
		$this->plugin_namespace = $this->plugin_info->get_text_domain();
	}

	public function init(): void {
		$this->settings_url = admin_url( 'admin.php?page=wc-settings&tab=checkout&section=wppay' );
		$this->support_url  = 'https://wppay.pl/pomoc/';

		require_once __DIR__ . "/../vendor_prefixed/guzzlehttp/guzzle/src/functions_include.php";
		require_once __DIR__ . "/../vendor_prefixed/guzzlehttp/promises/src/functions_include.php";
		require_once __DIR__ . "/../vendor_prefixed/guzzlehttp/psr7/src/functions_include.php";

		parent::init();
	}

	public function hooks(): void {
		parent::hooks();

		$plugin_url = $this->get_plugin_url();

		$client_factory = new BlueMediaClientFactory();
		$gateway        = new StandardPaymentGateway( $client_factory, $plugin_url );
		$client_factory->set_settings( $gateway );
		$gateway->hooks();

		( new BlikZeroEmbedGateway( $client_factory, $plugin_url ) )->hooks();
		/**
		 * @deprecated
		 */
		//( new CardEmbedGateway( $client_factory, $plugin_url ) )->hooks();
	}

	/**
	 * For external use.
	 *
	 * @param string $currency
	 *
	 * @return bool
	 */
	public static function has_credentials( string $currency = 'PLN' ): bool {
		$client_factory = new BlueMediaClientFactory();
		$client_factory->set_settings( new StandardPaymentGateway( $client_factory, '' ) );

		return $client_factory->has_credentials( $currency );
	}

	/**
	 * Admin enqueue scripts
	 */
	public function admin_enqueue_scripts() {
		$current_screen = get_current_screen();
		$suffix         = ''; //defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
		if ( in_array( $current_screen->id, [ 'woocommerce_page_wc-settings' ] )
		     && isset( $_GET['tab'] ) && $_GET['tab'] === 'checkout'
		     && isset( $_GET['section'] ) && $_GET['section'] === 'wppay'
		) {
			wp_register_style( 'wppay_admin_css', $this->get_plugin_assets_url() . 'css/admin' . $suffix . '.css', [],
				$this->scripts_version );
			wp_enqueue_style( 'wppay_admin_css' );
			wp_enqueue_script( 'wppay_admin_js', $this->get_plugin_assets_url() . 'js/admin' . $suffix . '.js',
				[ 'jquery' ], $this->scripts_version, true );
			$protocol = is_ssl() ? 'https://' : 'http://';
			wp_localize_script( 'wppay_admin_js', 'wppay_admin_object', [
				'site_url'    => str_replace( $protocol, '', site_url() ),
				'protocol'    => $protocol,
				'wppay_nonce' => wp_create_nonce( self::NONCE_AJAX_ACTION_GET_SINGLE_CURRENCY ),
			] );

			wp_register_style( 'jquery-ui-style', WC()->plugin_url() . '/assets/css/jquery-ui/jquery-ui.min.css',
				array(), $this->scripts_version );
			wp_enqueue_script( 'jquery-ui-accordion' );
		}
	}
}
