<?php

namespace WPDesk\GatewayWPPay\WooCommerceGateway;

use WPDesk\GatewayWPPay\BlueMediaApi\BlueMediaClientFactory;

final class BlikZeroEmbedGateway extends AbstractEmbedGateway {
	use RefundTrait;

	private const ID = 'wppay-blik-zero';

	private const SESSION_BLIK_PAYED        = 'blik-payed';
	private const SESSION_BLIK_TIME_STARTED = 'blik_started';
	private const JS_BLIK_TRIGGER           = 'wppay_blik_trigger';

	private const FORM_BLIK_CODE_INPUT_NAME = 'blik_code';

	private $plugin_url;

	public function __construct( BlueMediaClientFactory $client_factory, string $plugin_url ) {
		$this->plugin_url     = $plugin_url;
		$this->client_factory = $client_factory;

		$this->id                 = self::ID;
		$this->icon               = $plugin_url . '/assets/images/icon-blik.png';
		$this->has_fields         = true;
		$this->method_title       = __( 'Szybkie Płatności WP PAY - BLIK', 'woocommerce-gateway-wp-pay' );
		$this->method_description = __( 'Płatności online WP PAY - BLIK dla WooCommerce. Płatność bezpośrednio z formularza zamówienia, bez przekierowań.',
			'woocommerce-gateway-wp-pay' );

		$this->init_form_fields();
		$this->init_settings();

		$this->enabled     = $this->get_option( 'enabled', 'no' );
		$this->description = $this->get_option( 'description', 'BLIK dla WooCommerce.' );

		$this->title = $this->get_option( 'title',
			__( 'BLIK - zapłać przy użyciu kodu', 'woocommerce-gateway-wp-pay' ) );

		$this->supports = [
			'products',
			'refunds',
		];
	}

	public function hooks(): void {
		parent::hooks();
		if ( isset( $_POST['payment_method'] ) && $_POST['payment_method'] === self::ID ) {
			add_action( 'woocommerce_after_checkout_validation', [ $this, 'after_checkout_valudation' ] );
		}
		add_action( 'wc_ajax_wppay_refresh_blik_order_status', [ $this, 'refresh_blik_order_status' ], 10, 1 );
		add_action( 'wp_enqueue_scripts', [ $this, 'embed_blik_widget_js' ] );
	}

	public function after_checkout_valudation(): void {
		if ( empty( $_POST[ self::FORM_BLIK_CODE_INPUT_NAME ] ) ) {
			wc_add_notice( __( "Oczekiwanie na kod BLIK.", 'woocommerce-gateway-wp-pay' ), 'error' );
		}
	}

	public function process_payment( $order_id ): void {
		$blik_status = get_post_meta( $order_id, self::SESSION_BLIK_PAYED, true );
		if ( $blik_status !== 'SUCCESS' && ! empty( $_POST[ self::FORM_BLIK_CODE_INPUT_NAME ] ) ) {
			$order = wc_get_order( $order_id );
			$order->update_meta_data( self::SESSION_BLIK_TIME_STARTED, time() );
			$order->update_meta_data( self::SESSION_BLIK_PAYED, 'START' );
			$order->save();

			if ( $this->send_blik_payment( (int) $_POST[ self::FORM_BLIK_CODE_INPUT_NAME ], (int) $order_id ) ) {
				wc_add_notice( self::JS_BLIK_TRIGGER . $order_id, 'error' );
				wp_send_json( [
					'orderId'  => $order_id,
					'result'   => 'failure',
					'messages' => wc_print_notices( true ),
					'refresh'  => false,
					'reload'   => false,
				] );
			}
		}
	}

	public static function update_blik_status( int $order_id, string $status ): void {
		$order = wc_get_order( $order_id );
		$order->update_meta_data( self::SESSION_BLIK_PAYED, $status );
		$order->update_meta_data( self::SESSION_BLIK_PAYED, 'START' );
		$order->save();
	}

	private function send_blik_payment( int $code, int $order_id ): bool {
		$currency = get_woocommerce_currency();
		$client   = $this->client_factory->get_client( $currency );
		$order    = new \WC_Order( $order_id );

		$transaction_data = [
			'orderID'           => $order_id,
			'amount'            => $order->get_total( 'api' ),
			'description'       => apply_filters( 'wp_pay\payment\description',
				__( 'Transakcja WP Pay', 'woocommerce-gateway-wp-pay' ), $order, $this ),
			'currency'          => $currency,
			'gatewayID'         => 509,
			'customerEmail'     => $order->get_billing_email(),
			// Identyfikator usługi Blik
			'authorizationCode' => $code,
			'blikUIDKey'        => get_current_user_id(),
			// Klucz Aliasu UID (używane w BLIK). Jest to unikalny identyfikator użytkownika w Serwisie.
			//			'BlikAMKey' => '', // Klucz Aliasu aplikacji mobilnej banku (używane w BLIK). Jest to unikalny identyfikator konta w BLIK.
			// Etykieta Aliasu UID (używane w BLIK), która będzie prezentowana Klientowi w aplikacji bankowej w celu rozróżniania kont u Partnera.
			'blikUIDLabel'      => get_current_user_id(),

		];

		$response = $client->doTransactionInit( $transaction_data );
		/** @var \WPPayVendor\BlueMedia\Transaction\ValueObject\TransactionInit $data */
		$data = $response->getData();
		$data = $data->toArray();
		if ( in_array( $data['confirmation'], [ 'PENDING', 'SUCCESS' ] ) ) {
			$order->update_meta_data( self::SESSION_BLIK_PAYED, $data['confirmation'] );
			$order->save();
		}
		if ( isset( $data['reason'] ) ) {
			$order->update_meta_data( self::SESSION_BLIK_PAYED, $data['reason'] );
			$order->save();
		}

		return true;
	}

	public function embed_blik_widget_js(): void {
		if ( $this->enabled === 'yes' && is_checkout() ) {
			wp_enqueue_script( 'wppay-blik-script', $this->plugin_url . 'assets/js/blik-checkout.js', [], 16, false );
		}
	}

	public function refresh_blik_order_status(): void {
		$order_id = (int) $_POST['order_id'];
		if ( empty( $order_id ) ) {
			echo json_encode( [
				'type' => 'error',
				'msg'  => __( "Błąd podczas weryfikacji statusu BLIK.", 'woocommerce-gateway-wp-pay' ),
			] );
			wp_die();
		}

		$started = get_post_meta( $order_id, self::SESSION_BLIK_TIME_STARTED, time() );
		if ( $started <= time() - 3 * 60 ) {
			echo json_encode( [
				'type' => 'error',
				'msg'  => __( "Kod przedawnił się.", 'woocommerce-gateway-wp-pay' ),
			] );
			wp_die();
		}

		$blik_status = get_post_meta( $order_id, self::SESSION_BLIK_PAYED, true );
		if ( $blik_status === 'SUCCESS' ) {
			echo json_encode( [
				'type'     => 'success',
				'redirect' => $this->get_return_url( new \WC_Order( $order_id ) ),
			] );
		} elseif ( $blik_status === 'WRONG_TICKET' ) {
			echo json_encode( [
				'type' => 'error',
				'msg'  => __( "Nieprawidłowy kod.", 'woocommerce-gateway-wp-pay' ),
			] );
		} elseif ( $blik_status === 'MULTIPLY_PAID_TRANSACTION' ) {
			echo json_encode( [
				'type' => 'error',
				'msg'  => __( "Transakcja została już opłacona.", 'woocommerce-gateway-wp-pay' ),
			] );
		} elseif ( $blik_status === 'TICKET_EXPIRED' ) {
			echo json_encode( [
				'type' => 'error',
				'msg'  => __( "Kod przedawnił się.", 'woocommerce-gateway-wp-pay' ),
			] );
		} elseif ( $blik_status === 'START' || $blik_status === 'PENDING' ) {
			echo json_encode( [
				'type' => 'pending',
				'time' => $started,
				'msg'  => __( "Potwierdź kod w swojej aplikacji bankowej.", 'woocommerce-gateway-wp-pay' ),
			] );
		} else {
			echo json_encode( [
				'type' => 'error',
				'msg'  => __( "Nieprawidłowy kod.", 'woocommerce-gateway-wp-pay' ),
			] );
		}

		wp_die();
	}

	public function payment_fields(): void {
		ob_start();
		include __DIR__ . '/../views/wppay-blik-zero-form.php';
		echo ob_get_clean();
	}

}
